local mp = require 'mp'
local utils = require 'mp.utils'

local user = "kodi"
local password = "kodi"
local port = "8080"
local watchedPercentage = 80
local checkInterval = 10 -- in seconds

local kodiUrl = "http://"..user..":"..password.."@localhost:"..port.."/jsonrpc"
local KODI_GET_ITEM = '{"jsonrpc": "2.0", "method": "Player.GetItem", "params": { "playerid": 1 }, "id": "1"}'
local KODI_MARK_WATCHED_VIDEO = '{"jsonrpc": "2.0", "method": "VideoLibrary.SetMovieDetails", "params": { "movieid": $itemId, "playcount": 1 }, "id": "1"}'
local KODI_MARK_WATCHED_EPISODE = '{"jsonrpc": "2.0", "method": "VideoLibrary.SetEpisodeDetails", "params": { "episodeid": $itemId, "playcount": 1 }, "id": "1"}'
local KODI_REFRESH_VIDEO = '{"jsonrpc": "2.0", "method": "VideoLibrary.RefreshMovie", "params": { "movieid": $itemId }, "id": "1"}'
local KODI_REFRESH_EPISODE = '{"jsonrpc": "2.0", "method": "VideoLibrary.RefreshEpisode", "params": { "episodeid": $itemId }, "id": "1"}'

local timer

function on_start(event)
	timer = mp.add_periodic_timer(checkInterval, checkPlayStatus)
end

function checkPlayStatus()
	if mp.get_property_number("percent-pos") >= watchedPercentage then
		markWatched()
	end
end

function markWatched()
	local respGetItem = sendMessage(KODI_GET_ITEM)

	local msg = "Error sending command to Kodi"
	if respGetItem ~= nil and respGetItem.id == "1" then
		local itemId = respGetItem.result.item.id
		local type = respGetItem.result.item.type

		local markWatchedMsg
		if 'movie' == type then
			markWatchedMsg = string.gsub(KODI_MARK_WATCHED_VIDEO, "$itemId", itemId)
		elseif 'episode' == type then
			markWatchedMsg = string.gsub(KODI_MARK_WATCHED_EPISODE, "$itemId", itemId)
		end
		local respMarkWatched = sendMessage(markWatchedMsg)

		if (respMarkWatched ~= nil and respMarkWatched.result == "OK") then
			stopTimer()

			local refreshMsg
			if 'movie' == type then
				refreshMsg = string.gsub(KODI_REFRESH_VIDEO, "$itemId", itemId)
			elseif 'episode' == type then
				refreshMsg = string.gsub(KODI_REFRESH_EPISODE, "$itemId", itemId)
			end
			local respRefresh = sendMessage(refreshMsg)

			if (respRefresh ~= nil and respRefresh.result == "OK") then
				msg = "Marked as watched"
			end
		end
	end
	print(msg)
end

function stopTimer()
	if timer ~= nil and timer:is_enabled() then
		timer:kill()
		timer = nil
	end

end

function sendMessage(msg)
	local request = mp.command_native({
        name = "subprocess",
        capture_stdout = true,
        capture_stderr = true,
        playback_only = false,
        args = {"curl",
        		"-H", "Content-Type: application/json",
        		"-d", msg,
        		kodiUrl}
	})

	local response
	if request ~= nil then
		response = utils.parse_json(request.stdout)
	end
	return response
end

mp.register_event("file-loaded", on_start)
mp.observe_property("percent-pos", "number", on_percent_pos)
